#include "stdafx.h"

#include <iostream>
#include <fstream>
#include <string>
#include <vector>

#include "Net.h"

//The global values for the 2 rectangles
//Client on the left, server on the right
GLfloat clt_x1 = -0.75f, clt_y1 = 0.50f, clt_x2 = -0.25f, clt_y2 = -0.50f;
GLfloat svr_x1 = 0.25f, svr_y1 = 0.50f, svr_x2 = 0.75f, svr_y2 = -0.50f;
GLfloat dx = 0.05f, dy = 0.05f;

using namespace std;
using namespace net;

#ifdef DEBUG
#define check assert
#else
#define check(n) if ( !n ) { printf( "check failed\n" ); exit(1); }
#endif

const int ServerPort = 30000;
const int ClientPort = 30001;
const int ProtocolId = 0x11223344;
const float TimeOut = 100.0f;
const int PacketSize = 256;
const float DeltaTime = 0.001f;
float ticks;

ReliableConnection client( ProtocolId, TimeOut );

void decipherPacket(unsigned char packet[PacketSize])
{
	switch(packet[0] ) {
	case 'w':
		if(svr_y1 < 1.0f)
		{
			svr_y1 += dy;
			svr_y2 += dy;
		}
		break;
	case 's':
		if(svr_y2 > -1.0f)
		{
			svr_y1 -= dy;
			svr_y2 -= dy;
		}
		break;
	case 'a':
		if(svr_x1 > 0.0f)
		{
			svr_x1 -= dx;
			svr_x2 -= dx;
		}
		break;
	case 'd' :
		if(svr_x2 < 1.0f)
		{
			svr_x1 += dx;
			svr_x2 += dx;
		}		
		break;
	}
}

void setup(){
	// background is black
	glClearColor(1.0f, 0.0f, 0.0f, 0.0f);

}

void display(){
	if(client.IsConnected())
	{
		unsigned char client_packet[256];

		int bytes_read = client.ReceivePacket( client_packet, sizeof(client_packet) );
		if ( bytes_read != 0 )
		{
			cout << "server bytes received by client " << client_packet[0] << endl;
			decipherPacket(client_packet);
		}		
	}

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glColor3f(0.0f, 1.0f, 0.0f);
	glRectf(clt_x1, clt_y1, clt_x2, clt_y2); 
	glColor3f(0.0f, 0.0f, 1.0f); 
	glRectf(svr_x1, svr_y1, svr_x2, svr_y2);
	glFlush();
	glutSwapBuffers();
	
	
	if(++ticks > 250)
	{
		//heartbeat packet
		unsigned char packet[PacketSize];
		memset(packet, 'h', 1);
		client.SendPacket( packet, sizeof( packet ) );
		ticks = 0;
	}
	
}

void processNormalKeys(unsigned char key, int x, int y) {
	unsigned char packet[PacketSize];
	//Escape key
	if (key == 27)
	{

		// shutdown socket layer
		ShutdownSockets();
		exit(0);
	}

	else switch(key ) {

		case 'w':
			if(clt_y1 < 1.0f)
			{
				clt_y1 += dy;
				clt_y2 += dy;
				memset( packet, 'w', 1 );
			}
			break;
		case 's':
			if(clt_y2 > -1.0f)
			{
				clt_y1 -= dy;
				clt_y2 -= dy;
				memset( packet, 's', 1 );
			}
			break;
		case 'a':
			if(clt_x1 > -1.0f)
			{
				clt_x1 -= dx;
				clt_x2 -= dx;
				memset( packet, 'a', 1 );
			}
			break;
		case 'd' :
			if(clt_x2 < 0.0f)
			{
				clt_x1 += dx;
				clt_x2 += dx;
				memset( packet, 'd', 1 );
			}
			break;
	}
	client.SendPacket( packet, sizeof( packet ) );
}

int main(int argc, char *argv[]){
	//*************UDP
	// initialize

	if ( !InitializeSockets() )
	{
	printf( "failed to initialize sockets\n" );
	return 1;
	}

	printf( "-----------------------------------------------------\n" );
	printf( "test join\n" );
	printf( "-----------------------------------------------------\n" );

	check( client.Start( ClientPort ) );

	client.Connect( Address(127,0,0,1,ServerPort ) );
	
	while ( true )
	{
		if ( client.IsConnected() )
			break;

		if ( !client.IsConnecting() && client.ConnectFailed() )
			break;

		unsigned char client_packet[] = "client to server";
		client.SendPacket( client_packet, sizeof( client_packet ) );

		while ( true )
		{
			unsigned char packet[256];
			int bytes_read = client.ReceivePacket( packet, sizeof(packet) );
			if ( bytes_read == 0 )
				break;
		}

		client.Update( DeltaTime );
	}

	check( client.IsConnected() );


	//*************GLUT
	glutInit(&argc, argv);
	//GLUT_Double is required to enable a redraw
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH ); 
	glutInitWindowSize(800, 600);    // horizontal, vertical
	glutInitWindowPosition(500, 400);    // horizontal, vertical
	glutCreateWindow("Client");    // title of the window

	glutKeyboardFunc(processNormalKeys);  //Process the key input

	glutDisplayFunc(display);
	glutIdleFunc( display );
	glutMainLoop();
	return 0;
}