#include "stdafx.h"

#include <iostream>
#include <fstream>
#include <string>
#include <vector>

#include "Net.h"

//The global values for the 2 rectangles
//Client on the left, server on the right
GLfloat clt_x1 = -0.75f, clt_y1 = 0.50f, clt_x2 = -0.25f, clt_y2 = -0.50f;
GLfloat svr_x1 = 0.25f, svr_y1 = 0.50f, svr_x2 = 0.75f, svr_y2 = -0.50f;
GLfloat dx = 0.05f, dy = 0.05f;

using namespace std;
using namespace net;

#ifdef DEBUG
#define check assert
#else
#define check(n) if ( !n ) { printf( "check failed\n" ); exit(1); }
#endif

const int ServerPort = 30000;
const int ClientPort = 30001;
const int ProtocolId = 0x11223344;
const float TimeOut = 100.0f;
const int PacketSize = 256;

//const int ProtocolId = 0x11112222;
const float DeltaTime = 0.001f;
//const float TimeOut = 1.0f;

ReliableConnection server( ProtocolId, TimeOut );

void decipherPacketAndSend(unsigned char packet)
{
	unsigned char server_packet[PacketSize];

	switch(packet ) {
	case 'h':
		//heartbeat
		break;
	
	case 'w':
		if(svr_y1 < 1.0f)
		{
			svr_y1 += dy;
			svr_y2 += dy;
			memset( server_packet, 'w', 1 );
		}
		break;
	case 's':
		if(svr_y2 > -1.0f)
		{
			svr_y1 -= dy;
			svr_y2 -= dy;
			memset( server_packet, 's', 1 );
		}
		break;
	case 'a':
		if(svr_x1 > -1.0f)
		{
			svr_x1 -= dx;
			svr_x2 -= dx;
			memset( server_packet, 'a', 1 );
		}
		break;
	case 'd' :
		if(svr_x2 < 1.0f)
		{
			svr_x1 += dx;
			svr_x2 += dx;
			memset( server_packet, 'd', 1 );
		}
		break;
	}
	if(server_packet[0] == 'w' || server_packet[0] == 's' || server_packet[0] == 'a' || server_packet[0] == 'd')
		server.SendPacket( server_packet, sizeof(server_packet) );
	else
	{
		memset(server_packet, 'n', 1);
		server.SendPacket( server_packet, sizeof(server_packet) );
	}
}




int main(int argc, char *argv[]){
	//*************UDP

	if ( !InitializeSockets() )
	{
		printf( "failed to initialize sockets\n" );
		return 1;
	}

	printf( "-----------------------------------------------------\n" );
	printf( "test join\n" );
	printf( "-----------------------------------------------------\n" );

	check( server.Start( ServerPort ) );
	server.Listen();

	while ( true )
	{
		if ( server.IsConnected() )
			break;

		unsigned char server_packet[] = "server to client";
		server.SendPacket( server_packet, sizeof( server_packet ) );

		while ( true )
		{
			unsigned char packet[256];
			int bytes_read = server.ReceivePacket( packet, sizeof(packet) );
			if ( bytes_read == 0 )
				break;
		}

		server.Update( DeltaTime );
	}

	check( server.IsConnected() );

	while(true)
	{
		if(server.IsConnected())
		{
			unsigned char server_packet[256];	
			int bytes_read;
			bytes_read = server.ReceivePacket( server_packet, sizeof(server_packet) );
			if ( bytes_read != 0 )
			{
				cout << "client bytes received by server " << server_packet[0] << endl;
				decipherPacketAndSend(server_packet[0]);
			}		
		}
		//This update is super important!! otherwise it crashes
		server.Update(DeltaTime);
	}
	
	return 0;
}